/* This software is licensed by the Apache License 2.0, see LICENSE file */
/*                                       Copyright © 2022 Gregory Lirent */

#include "vnode.h"

#ifndef LIBCDSB_SRC_INTERNAL_RBTREE_H
#define LIBCDSB_SRC_INTERNAL_RBTREE_H

typedef struct libcdsb_rbtree_node {
    struct libcdsb_rbtree_node* left;
    struct libcdsb_rbtree_node* right;
    struct libcdsb_rbtree_node* parent;

    vnode_t value;
    short colored;
} rbnode_t;

extern rbnode_t LIBCDSB_BUILTIN_RBTREE_NODE_EMPTY[1];

extern void*     libcdsb_builtin_rbtree_node_create(void* value, rbnode_t* parent, int colored, int size) Nonnull__(  2);
extern void      libcdsb_builtin_rbtree_node_fixup (rbnode_t** root, rbnode_t* node)                      Nonnull__(1,2);
extern rbnode_t* libcdsb_builtin_rbtree_node_delete(rbnode_t** root, rbnode_t* node)                      Nonnull__(1,2);

extern stack_t libcdsb_builtin_rbtree_iter_inorder      (rbnode_t** root, bool reverse);
extern stack_t libcdsb_builtin_rbtree_iter_preorder     (rbnode_t** root, bool reverse);
extern stack_t libcdsb_builtin_rbtree_iter_postorder    (rbnode_t** root, bool reverse);
extern stack_t libcdsb_builtin_rbtree_iter_breadth_first(rbnode_t** root, bool reverse);

#define rbnode_empty           ((rbnode_t*)LIBCDSB_BUILTIN_RBTREE_NODE_EMPTY)
#define rbnode_create(v, p, c) ((rbnode_t*)libcdsb_builtin_rbtree_node_create(v, p, c, sizeof(rbnode_t)))
#define rbnode_fixup           libcdsb_builtin_rbtree_node_fixup
#define rbnode_delete          libcdsb_builtin_rbtree_node_delete

#define rbnode_is_empty(n) ((n) == rbnode_empty)
#define rbnode_is_root(n)  rbnode_is_empty((n)->parent)

#define rbiter_inorder       libcdsb_builtin_rbtree_iter_inorder
#define rbiter_preorder      libcdsb_builtin_rbtree_iter_preorder
#define rbiter_postorder     libcdsb_builtin_rbtree_iter_postorder
#define rbiter_breadth_first libcdsb_builtin_rbtree_iter_breadth_first

#endif /* LIBCDSB_SRC_INTERNAL_RBTREE_H */
