/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../modules/libunic/include.h"
#include "__internal/include.h"

static _Thread_local int  LIBCDSB_BUILTIN_COUNTER = 0;
static _Thread_local char LIBCDSB_BUILTIN_BUFFER[16][5];

const char* libcdsb_char_to_cstring(int c) {

    char* p;

    if (LIBCDSB_BUILTIN_COUNTER > 15)
        LIBCDSB_BUILTIN_COUNTER = 0;

    if (is_null(p = tochar_unicode(LIBCDSB_BUILTIN_BUFFER[LIBCDSB_BUILTIN_COUNTER], c))) {
        LIBCDSB_BUILTIN_BUFFER[LIBCDSB_BUILTIN_COUNTER][0] = 0;
    } else *p = 0;

    return LIBCDSB_BUILTIN_BUFFER[LIBCDSB_BUILTIN_COUNTER++];
}


size_t libcdsb_strlen(const char* s) {
    static const size_t m = (sizeof(size_t) == 8) ? 0x8080808080808080UL : 0x80808080UL;
    static const size_t d = (sizeof(size_t) == 8) ? 0x0101010101010101UL : 0x01010101UL;

    const u8_t* p;
    const size_t* w;

    p = (void*)s;

    while((uintptr_t)p%sizeof(size_t)) {
        if (!*p) return (void*)p - (void*)s;
        ++p;
    }

    w = (void*)p;

    while (!((*w-d)&(~*w&m))) { ++w; }

    p = (void*)w;

    while (*p) { ++p; }

    return (void*)p - (void*)s;
}


size_t libcdsb_strasciilen(const char* s) {
    static const size_t m = (sizeof(size_t) == 8) ? 0x8080808080808080UL : 0x80808080UL;
    static const size_t d = (sizeof(size_t) == 8) ? 0x0101010101010101UL : 0x01010101UL;

    const u8_t* p;
    const size_t* w;

    p = (void*)s;

    while((uintptr_t)p%sizeof(size_t)) {
        if (!*p || (*p&0x80))
            return (void*)p - (void*)s;
        ++p;
    }

    w = (void*)p;

    while (!((*w&m)||((*w-d)&m))) { ++w; }

    p = (void*)w;

    while (*p && !(*p&0x80)) { ++p; }

    return (void*)p - (void*)s;
}
