/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

hash_t dict_hash(const dict_t* s) {
    dnode_t *min, *max;
    size_t           i;
    dnode_t         *c;
    hash_t        hash;

    if (!s->size)
        return 0;

    min = max = nullptr;
    i   = s->capacity;

    while (i--) {
        c = s->nodes[i];

        while (!is_null(c)) {
            if (is_null(min) || vnode_compare(&c->key, c->key_type, &min->key, min->key_type) < 0) min = c;
            if (is_null(max) || vnode_compare(&c->key, c->key_type, &max->key, max->key_type) > 0) max = c;

            c = c->prev;
        }
    }

    hash = vnode_hash(&min->key, min->key_type);

    if (s->size > 0)
        hash += vnode_hash(&max->key, max->key_type);

    return (hash ^ s->size) + VTYPE_DICT;
}
