/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

static int libcdsb_builtin_foreach(map_t* x, void* data, map_access_callback callback) {
    stack_t z, *top, *bot, *cur;
    mnode_t* n;
    int  r = 0;

    memset(&z, 0, sizeof(z));

    if (rbnode_is_empty(z.value = x->root))
        return 0;

    for (top = bot = &z;;) {
        for (cur = bot;;) {

            n = top->value;

            if (!mnode_is_empty(n->left))  cur = stack_insert(cur, n->left);
            if (!mnode_is_empty(n->right)) cur = stack_insert(cur, n->right);

            if (!r) {
                r = callback(vnode_peek(&n->key, x->type), x->type, vnode_peek(&n->value, n->type), n->type, data);
            } else {
                stack_flush(&z);
                return r;
            }

            if (top == bot) {
                top = top->prev;
                break;
            } else top = top->prev;
        }

        if (!is_null(top)) {
            while (!is_null(bot->prev)) bot = bot->prev;
        } else break;
    }

    stack_flush(&z);
    return r;
}

/*#####################################################################################################################*/

int libcdsb_map_find(map_t* x, const void* k, vtype t, void* _, map_access_callback callback, bool cut) {
    mnode_t*  c;
    void   *key;
    int     cmp;

    c = x->root;

    while (!mnode_is_empty(c)) {
        key = vnode_peek(&c->key, x->type);
        cmp = vtype_compare(k, t, key, x->type);

        if (cmp == 0) {
            cmp = (callback) ? callback(key, x->type, vnode_peek(&c->value, c->type), c->type, _) : 0;

            if (cut) {
                c = mnode_delete(&x->root, c);
                vnode_free(&c->key,   x->type);
                vnode_free(&c->value, c->type);
                free(c);
            }

            return cmp;
        } else c = (cmp < 0) ? c->left : c->right;
    }

    return -1;
}


int libcdsb_map_foreach(map_t* x, void* data, map_access_callback callback, rbforeach_t type, bool flush) {
    bool reverse;
    stack_t iter;
    mnode_t*   n;
    int    r = 0;

    reverse = type&RBFOREACH_REVERSE;

    switch (type&(RBFOREACH_INORDER|RBFOREACH_PREORDER|RBFOREACH_POSTORDER|RBFOREACH_BREADTH_FIRST)) {
        case RBFOREACH_INORDER:       iter = miter_inorder  (&x->root, reverse); break;
        case RBFOREACH_PREORDER:      iter = miter_preorder (&x->root, reverse); break;
        case RBFOREACH_POSTORDER:     iter = miter_postorder(&x->root, reverse); break;

        default:
        case RBFOREACH_BREADTH_FIRST: if (reverse || flush) {
                                          iter = miter_breadth_first(&x->root, type&RBFOREACH_REVERSE);
                                          break;
                                      } else return libcdsb_builtin_foreach(x, data, callback);
    }

    while ((n = stack_pop(&iter))) {
        if (!r) {
            r = callback(vnode_peek(&n->key, x->type), x->type, vnode_peek(&n->value, n->type), n->type, data);
        } else if (!flush) {
            stack_flush(&iter);
            return r;
        }

        if (flush) {
            vnode_free(&n->value, x->type);
            free(n);
        }
    }

    if (flush) x->root = mnode_empty;

    return r;
}
