/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

static inline int libcdsb_builtin_compare(const mnode_t* s0, const mnode_t* s1, vtype t) {
    int c = vnode_compare_eq(&s0->key, &s1->key, t);

    return !c ? vnode_compare(&s0->value, s0->type, &s1->value, s1->type) : c;
}

/*#####################################################################################################################*/

int map_compare(const map_t* s0, const map_t* s1) {
    stack_t        z;
    mnode_t *c0, *c1;
    int          cmp;

    if (s0 == s1 || s0->root == s1->root)
        return 0;
    if (s0->type != s1->type)
        return s0->type - s1->type;

    stack_init(&z);
    stack_push_many(&z, 2, (void*)s1, (void*)s0);

    cmp = 0;

    do {
        c0 = stack_pop(&z);
        c1 = stack_pop(&z);

        if (mnode_is_empty(c0) || mnode_is_empty(c1)) {
            if (c0 != c1) {
                stack_flush(&z);
                return mnode_is_empty(c0) ? -1 : 1;
            }
        } else if ((cmp = libcdsb_builtin_compare(c0, c1, s0->type))) {
            if (c0->left == c1->right) {  // == mnode_empty
                          cmp = libcdsb_builtin_compare(c0->right, c1, s0->type);
                if (!cmp) cmp = libcdsb_builtin_compare(c0, c1->left, s0->type);
            } else if (c0->right == c1->left) { // == mnode_empty
                          cmp = libcdsb_builtin_compare(c0, c1->right, s0->type);
                if (!cmp) cmp = libcdsb_builtin_compare(c0->left, c1, s0->type);
            }

            if (cmp) {
                stack_flush(&z);
                return cmp;
            }
        } else stack_push_many(&z, 4, c1->right, c0->right, c1->left, c0->left);

    } while (!is_null(z.value));

    return 0;
}
