/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../../include/map.h"
#include "../__internal/assert.h"
#include "../__internal/rbtree.h"

#ifndef LIBCDSB_SRC_MAP_INCLUDE_H
#define LIBCDSB_SRC_MAP_INCLUDE_H

typedef struct libcdsb_map_node {
    struct libcdsb_map_node* left;
    struct libcdsb_map_node* right;
    struct libcdsb_map_node* parent;

    vnode_t key;
    short   colored;
    short   type;
    vnode_t value;
} mnode_t;

static_assert(offsetof(struct libcdsb_rbtree_node, left)    == offsetof(struct libcdsb_map_node, left),    "Implementation assert");
static_assert(offsetof(struct libcdsb_rbtree_node, right)   == offsetof(struct libcdsb_map_node, right),   "Implementation assert");
static_assert(offsetof(struct libcdsb_rbtree_node, parent)  == offsetof(struct libcdsb_map_node, parent),  "Implementation assert");
static_assert(offsetof(struct libcdsb_rbtree_node, colored) == offsetof(struct libcdsb_map_node, colored), "Implementation assert");
static_assert(offsetof(struct libcdsb_rbtree_node, value)   == offsetof(struct libcdsb_map_node, key),     "Implementation assert");
static_assert(offsetof(struct libcdsb_set, root)            == offsetof(struct libcdsb_map, root),         "Implementation assert");
static_assert(offsetof(struct libcdsb_set, type)            == offsetof(struct libcdsb_map, type),         "Implementation assert");

#define mnode_empty           ((mnode_t*)rbnode_empty)
#define mnode_create(k, p, c) ((mnode_t*)libcdsb_builtin_rbtree_node_create(k, (rbnode_t*)p, c, sizeof(mnode_t)))
#define mnode_fixup(r, n)     libcdsb_builtin_rbtree_node_fixup((rbnode_t**)(r), (rbnode_t*)(n))
#define mnode_delete(r, n)    (mnode_t*)libcdsb_builtin_rbtree_node_delete((rbnode_t**)(r), (rbnode_t*)(n))

#define mnode_is_empty(n) ((n) == mnode_empty)
#define mnode_is_root(n)  mnode_is_empty((n)->parent)

#define miter_inorder(x, reverse)       rbiter_inorder((void*)x, reverse)
#define miter_preorder(x, reverse)      rbiter_preorder((void*)x, reverse)
#define miter_postorder(x, reverse)     rbiter_postorder((void*)x, reverse)
#define miter_breadth_first(x, reverse) rbiter_breadth_first((void*)x, reverse)

#endif /* LIBCDSB_SRC_MAP_INCLUDE_H */
