/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include <stdarg.h>
#include <stdlib.h>
#include "__internal/include.h"
#undef malloc
#undef free

void libcdsb_queue_init(queue_t* x) {
    memset(x, 0, sizeof(*x));
}


void libcdsb_queue_push(queue_t* x, void* value) {
    stack_t* n;

    if (!(n = malloc(sizeof(*n))))
        abort();

    n->prev = 0;

    if (!x->back) {
        x->front = n;
        x->back  = n;
    } else x->back->prev = n;

    n->value = value;
}


void libcdsb_queue_push_many(queue_t* x, size_t c, ...) {
    va_list args;
    stack_t* n;
    va_start(args, c);

    if (c) {

        if (!x->back) {
            if (!(n = malloc(sizeof(*n))))
                abort();

            x->front = n;
            x->back  = n;
            n->prev  = 0;
            n->value = va_arg(args, void*);
            --c;
        }

        while (c--) {
            if (!(n = malloc(sizeof(*n))))
                abort();

            x->back->prev = n;
            n->prev       = 0;
            n->value      = va_arg(args, void*);
        }
    }

    va_end(args);
}


void* libcdsb_queue_pop(queue_t* x) {
    stack_t* n;
    void*    v;

    if (x->front) {
        n = x->front;
        v = n->value;

        if (!(x->front = n->prev)) {
            x->back = 0;
        }

        free(n);
    } else v = 0;

    return v;
}


void libcdsb_queue_flush(queue_t* x) {
    stack_t* c;

    c = x->front;

    while (x->front) {
        c = x->front;
        x->front = c->prev;
        free(c);
    }

    x->back = 0;
}
