/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../../include/set.h"
#include "../__internal/rbtree.h"

static inline int libcdsb_builtin_compare(const rbnode_t* s0, const rbnode_t* s1, vtype t) {
    return vnode_compare(s0->value, t, s1->value, t);
}

/*#####################################################################################################################*/

int vset_compare(const set_t* s0, const set_t* s1) {
    stack_t         z;
    rbnode_t *c0, *c1;
    int           cmp;

    if (s0 == s1 || s0->root == s1->root)
        return 0;
    if (s0->type != s1->type)
        return s0->type - s1->type;

    stack_init(&z);
    stack_push_many(&z, 2, (void*)s1, (void*)s0);

    cmp = 0;

    do {
        c0 = stack_pop(&z);
        c1 = stack_pop(&z);

        if (rbnode_is_empty(c0) || rbnode_is_empty(c1)) {
            if (c0 != c1) {
                stack_flush(&z);
                return rbnode_is_empty(c0) ? -1 : 1;
            }
        } else if ((cmp = libcdsb_builtin_compare(c0, c1, s0->type))) {
            if (c0->left == c1->right) {  // == rbnode_empty
                          cmp = libcdsb_builtin_compare(c0->right, c1, s0->type);
                if (!cmp) cmp = libcdsb_builtin_compare(c0, c1->left, s0->type);
            } else if (c0->right == c1->left) { // == rbnode_empty
                          cmp = libcdsb_builtin_compare(c0, c1->right, s0->type);
                if (!cmp) cmp = libcdsb_builtin_compare(c0->left, c1, s0->type);
            }

            if (cmp) {
                stack_flush(&z);
                return cmp;
            }
        } else stack_push_many(&z, 4, c1->right, c0->right, c1->left, c0->left);

    } while (!is_null(z.value));

    return 0;
}
