/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../../include/set.h"
#include "../__internal/rbtree.h"

void vset_init(set_t* x, vtype t) {
    x->root = rbnode_empty;
    x->type = t;
}

void vset_free(set_t* x) {
    rbnode_t *t, *c;

    if (is_null(x)) return;

    c = x->root;

    while (!rbnode_is_empty(x->root)) {
        if (!rbnode_is_empty(c->left)) {
            c = c->left;
        } else if (!rbnode_is_empty(c->right)) {
            c = c->right;
        } else if (!rbnode_is_root(c)) {
            vnode_free(&c->value, x->type);

            t = c;
            c = c->parent;

            if (t == c->left) c->left  = rbnode_empty;
            else              c->right = rbnode_empty;

            free(t);
        } else {
            vnode_free(&c->value, x->type);
            x->root = rbnode_empty;
        }
    }

    x->type = 0;
}
