/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

size_t string_slice(str_t* x, str_t* s, ssize_t i, size_t c, bool cut) {
    char *e, *p, *v;
    size_t n = 0;

    memset(x, 0, sizeof(*x));

    if (!c) return n;

    p = at_string(s, i);

    if (is_null(p))
        return n;

    e = p + strlen(p);
    v = p;

    while (c-- && v < e) {
        v = next_char(v);
        ++n;
    }

    x->buffer = strndup(p, v - p);

    if (cut) {
        memmove(p, v, strlen(v) + 1);
    }

    return n;
}


arr_t libcdsb_string_split(const str_t* x, const char* s, size_t sn, size_t maxn) {
    arr_t   ret;
    char *p, *e;
    str_t*    v;

    ret.mem  = 0;
    ret.size = 0;
    ret.type = VTYPE_STRING;

    if (is_null(x->buffer))
        return ret;

    if (is_null(s) || !*s) {
        v = ret.mem = malloc(sizeof(str_t));
        v->buffer   = strdup(x->buffer);

        return ret;
    }

    if (!sn) sn = strlen(s);
    p = x->buffer;
    e = p;

    while (maxn-- && !is_null(p = strstr(p, s))) {
        p += sn;
        v  = ret.mem = realloc(ret.mem, ++ret.size * sizeof(str_t));

        v[ret.size-1].buffer = strndup(e, p - e);

        p += sn;
        e  = p;
    }

    if (*e) {
        sn = strlen(e);
        v = ret.mem = realloc(ret.mem, ++ret.size*sizeof(str_t));

        v[ret.size-1].buffer = strndup(e, sn);
    }

    return ret;
}
