/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../plug.h"

static int array_value_print(void* v, ssize_t i, vtype t, void* _) {
    print_container_value(&i, v, t, 1, *(unsigned int*)_);
    return 0;
}

void array_print(arr_t* x, const char* prefix, unsigned int hpos) {
    print_container_values_prefix("Array", prefix, hpos);
    array_foreach(x, &hpos, array_value_print);
}

void array_info(arr_t* x, unsigned int hpos) {
    print_container_info("Array", "nodes", 0, array_size(x), -1, hpos);
}


void visual_push(arr_t* x, size_t n) {
    for (int i = 0; i < n; ++i) {
        fputs("\e[s", stdout);

        array_push_random(x, 0, 0);

        array_info(x, 0);
        array_print(x, 0, 0);

        psleep(100000);
        fputs("\e[u\e[J", stdout);
    }
}

void visual_sort(arr_t* x) {
    puts("\e[s\e[36mTry to sort array values:\e[m\n\n");
    put_separator(0);
    array_sort(x);
    array_info(x, 0);
    array_print(x, "sorted", 0);
    psleep(900000);
    fputs("\e[u\e[J", stdout);

    puts("\e[s\e[36mTry to reverse array values:\e[m\n\n");
    put_separator(0);
    array_reverse(x);
    array_info(x, 0);
    array_print(x, "reversed", 0);
    psleep(900000);
    fputs("\e[u\e[J", stdout);
}

void visual_remove(arr_t* x) {
    while (x->size) {
        fputs("\e[s", stdout);
        array_remove_random(x, 0, 0);
        array_info(x, 0);
        array_print(x, 0, 0);

        psleep(100000);
        if (!0) fputs("\e[u\e[J", stdout);
    }
}

void visual_push2(arr_t* x0, size_t n0, arr_t* x1, size_t n1) {

    for (;n0 || n1;) {
        fputs("\e[s", stdout);
        if (n0) {
            array_push_random(x0, 0, 0);
            --n0;
        } else {
            puts("\n\n");
            put_separator(0);
        }

        fputs("\e[u\e[s", stdout);

        if (n1) {
            array_push_random(x1, 0, 60);
            --n1;
        } else {
            puts("\n\n");
            put_separator(60);
        }

        array_info(x0, 0);
        array_print(x0, 0, 0);
        fputs("\e[u\e[s\e[4E", stdout);
        array_info(x1, 60);
        array_print(x1, 0, 60);
        psleep(100000);

        fputs("\e[u\e[J", stdout);
    }
}

void visual_remove2(arr_t* x0, arr_t* x1) {

    for (;x0->size || x1->size;) {
        fputs("\e[s", stdout);

        if (x0->size) {
            array_remove_random(x0, 0, 0);
        } else {
            puts("\n\n");
            put_separator(0);
        }

        fputs("\e[u\e[s", stdout);

        if (x1->size) {
            array_remove_random(x1, 0, 60);
        } else {
            puts("\n\n");
            put_separator(60);
        }

        array_info(x0, 0);
        array_print(x0, 0, 0);
        fputs("\e[u\e[s\e[4E", stdout);
        array_info(x1, 60);
        array_print(x1, 0, 60);
        psleep(100000);

        fputs("\e[u\e[J", stdout);
    }
}


void visual_sort2(arr_t* x0, arr_t* x1) {
    printf("\e[s\e[36m%-60s%s\e[m\n\n","Try to sort array values:", "Try to sort array values:");
    printf("\e[32;1m%-60s%s\e[m\n", "SUCCESS", "SUCCESS");
    printf("\e[37;2m%-60s%s\e[m\n", "=== === === === === === === ===", "=== === === === === === === ===");

    array_sort(x0);
    array_sort(x1);

    array_info(x0, 0);
    array_print(x0, "sorted", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    array_info(x1, 60);
    array_print(x1, "sorted", 60);

    psleep(900000);
    fputs("\e[u\e[J", stdout);

    printf("\e[s\e[36m%-60s%s\e[m\n\n","Try to reverse array values:", "Try to reverse array values:");
    printf("\e[32;1m%-60s%s\e[m\n", "SUCCESS", "SUCCESS");
    printf("\e[37;2m%-60s%s\e[m\n", "=== === === === === === === ===", "=== === === === === === === ===");

    array_reverse(x0);
    array_reverse(x1);

    array_info(x0, 0);
    array_print(x0, "reversed", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    array_info(x1, 60);
    array_print(x1, "reversed", 60);

    psleep(900000);
    fputs("\e[u\e[J", stdout);
}

void visual_slice(arr_t* x, arr_t* s) {

    size_t  n = array_size(x);
    ssize_t i = random_uint64()%(n - 1);

    n = n - (i + 1);

    puts("\e[s\e[36mTry to slice array:\e[m\n\n");
    printf("\e[37;2m%-60s%s\e[m\n", "=== === === === === === === ===", "=== === === === === === === ===");

    array_info(x, 0);
    array_print(x, "(src)", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    array_info(s, 60);
    array_print(s, "(dest)", 60);

    psleep(900000);

    array_slice(s, x, i, n, 1);
    fputs("\e[u\e[s\e[2E\e[32;1mSUCCESS\e[m\e[J", stdout);

    puts("");
    put_separator(0);

    array_info(x, 0);
    array_print(x, "(src)", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    array_info(s, 60);
    array_print(s, "(dest)", 60);
    psleep(900000);

    fputs("\e[u\e[J", stdout);
}
