/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../plug.h"

static int remove_callback(void* v, ssize_t i, vtype t, void* _) {
    struct { arr_t* x; _Bool s; unsigned int p; } *x = _;
    if (!x->s) print_container_value(0, v, t, 1, x->p);

    if (libcdsb_array_find(x->x, v, t, 0, 0, 1, 1))
        return -2;
    return 0;
}

static int change_callback(void* v, ssize_t i, vtype t, void* _) {
    struct { value_t v; _Bool s; unsigned int p; } *x = _;

    while (x->v.type != t) { x->v = random_value(); }

    memcpy(v, &x->v, vtype_size(t));
    return 0;
}

void array_push_random(arr_t* x, _Bool silent, unsigned int hpos) {

    struct { value_t v; _Bool s; unsigned int p; } _ = { .v = random_value(), .s = silent, .p = hpos };
    _Bool   r;

    if (random_boolean()) {
        if (!silent) {
            printf("\e[%dG\e[36mTry to push value to back of the array:\e[m\n", hpos+1);
        }

        r = libcdsb_array_insert(x, _.v.value, _.v.type) >= 0;
    } else {
        ssize_t i = array_size(x);

        i = random_uint64()  % (i ? i : 1);
        i = random_boolean() ? ~i + 1 : i ;

        if (!silent) {
            printf("\e[%dG\e[36mTry to change value in the array with index \e[32;1m%ld\e[36m:\e[m\n", hpos+1, i);
        }

        r = array_get_by_index(x, i, &_, change_callback) == 0;
    }

    if (!silent) {
        print_container_value(0, _.v.value, _.v.type, 1, hpos);
        printf("\e[%dG%s\n", hpos+1, r ? "\e[32;1mSUCCESS\e[m" : "\e[31;1mFAILURE\e[m");
        put_separator(hpos);
    }
}

void array_remove_random(arr_t* x, _Bool silent, unsigned int hpos) {

    size_t  n = array_size(x);
    ssize_t i = random_uint64()%n;

    if (random_boolean()) i = ~i + 1;

    if (random_boolean()) {
        if (!silent) {
            printf("\e[%dG\e[36mTry to remove value from list by index:\e[m\n", hpos+1);
            print_container_value(0, &i, (sizeof(ssize_t) == 8) ? VTYPE_INT64 : VTYPE_INT32, 0, hpos);
        }
        switch (array_remove_by_index(x, i)) {
            case  0: if (!silent) printf("\e[%dG\e[32;1mSUCCESS\e[m\n", hpos+1); break;
            default: if (!silent) printf("\e[%dG\e[32;1mFAILURE\e[m\n", hpos+1); break;
        }
    } else {
        struct { arr_t* x; _Bool s; unsigned int p; } v = { .x = x, .s = silent, .p = hpos };

        if (!silent) printf("\e[%dG\e[36mTry to remove value from list:\e[m\n", hpos+1);

        switch (libcdsb_array_get(x, i, &v, remove_callback, 0)) {
            case  0: if (!silent) printf("\e[%dG\e[32;1mSUCCESS\e[m\n", hpos+1); break;
            default: if (!silent) printf("\e[%dG\e[32;1mFAILURE\e[m\n", hpos+1); break;
        }
    }

    if (!silent) put_separator(hpos);
}
