/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../plug.h"

#define MAX_ELEMENTS 2000

static arr_t  random_array(bool embd);
static list_t random_list (bool embd);
static set_t  random_set  (bool embd);
static map_t  random_map  (bool embd);
static dict_t random_dict (bool embd);

static str_t random_string() {
    str_t  x;
    size_t n = random_uint16()%MAX_ELEMENTS;

    string_init(&x, 0);

    while (n--) {
        string_concat(&x, random_unicode_symbol());
    }

    return x;
}

static value_t random_value2() {
    value_t v;
    switch (random_uint8()%14) {
        default:
        case  0: v.value[0].b   = random_boolean(); v.type = VTYPE_BOOLEAN; break;
        case  1: v.value[0].u8  = random_uint8  (); v.type = VTYPE_UINT8;   break;
        case  2: v.value[0].u16 = random_uint16 (); v.type = VTYPE_UINT16;  break;
        case  3: v.value[0].u32 = random_uint32 (); v.type = VTYPE_UINT32;  break;
        case  4: v.value[0].u64 = random_uint64 (); v.type = VTYPE_UINT64;  break;
        case  5: v.value[0].u8  = random_int8   (); v.type = VTYPE_INT8;    break;
        case  6: v.value[0].u16 = random_uint16  (); v.type = VTYPE_INT16;   break;
        case  7: v.value[0].u32 = random_int32  (); v.type = VTYPE_INT32;   break;
        case  8: v.value[0].u64 = random_int64  (); v.type = VTYPE_INT64;   break;
        case  9: v.value[0].f   = random_float  (); v.type = VTYPE_FLOAT;   break;
        case 10: v.value[0].d   = random_double (); v.type = VTYPE_DOUBLE;  break;
        case 11: v.value[0].ld  = random_ldouble(); v.type = VTYPE_LDOUBLE; break;
        case 12: v.value[0].ptr = (void*)(uintptr_t)random_uint64(); v.type = VTYPE_POINTER; break;
        case 13: v.value[0].s   = random_string();  v.type = VTYPE_STRING;  break;
    }

    return v;
}

static value_t random_value_by_type(vtype type, bool embd) {
    value_t v;
    switch ((v.type = type)) {
        default:
        case VTYPE_BOOLEAN: v.value[0].b   = random_boolean();   break;
        case   VTYPE_UINT8: v.value[0].u8  = random_uint8  ();   break;
        case  VTYPE_UINT16: v.value[0].u16 = random_uint16 ();   break;
        case  VTYPE_UINT32: v.value[0].u32 = random_uint32 ();   break;
        case  VTYPE_UINT64: v.value[0].u64 = random_uint64 ();   break;
        case    VTYPE_INT8: v.value[0].u8  = random_int8   ();   break;
        case   VTYPE_INT16: v.value[0].u16 = random_uint16  ();   break;
        case   VTYPE_INT32: v.value[0].u32 = random_int32  ();   break;
        case   VTYPE_INT64: v.value[0].u64 = random_int64  ();   break;
        case   VTYPE_FLOAT: v.value[0].f   = random_float  ();   break;
        case  VTYPE_DOUBLE: v.value[0].d   = random_double ();   break;
        case VTYPE_LDOUBLE: v.value[0].ld  = random_ldouble();   break;
        case  VTYPE_STRING: v.value[0].s   = random_string();    break;
        case   VTYPE_ARRAY: v.value[0].a   = random_array(embd); break;
        case     VTYPE_MAP: v.value[0].m   = random_map  (embd); break;
        case    VTYPE_DICT: v.value[0].vd  = random_dict (embd); break;
        case    VTYPE_LIST: v.value[0].l   = random_list (embd); break;
        case     VTYPE_SET: v.value[0].vs  = random_set  (embd); break;
        case VTYPE_POINTER: v.value[0].ptr = (void*)(uintptr_t)random_uint64(); break;
    }

    return v;
}

static arr_t random_array(bool embd) {
    arr_t   x;
    size_t  n = random_uint16()%((!embd)?MAX_ELEMENTS:100);
    value_t v = (!embd) ? random_value2() : real_random_value(1);

    array_init(&x, v.type);

    while(n--) {

        libcdsb_array_insert(&x, v.value, v.type);

        switch (v.type) {
            default: break;
            case VTYPE_STRING: string_free((void*)v.value); break;
            case  VTYPE_ARRAY: array_free((void*)v.value);  break;
            case   VTYPE_LIST: list_free((void*)v.value);   break;
            case    VTYPE_MAP: map_free((void*)v.value);    break;
            case    VTYPE_SET: vset_free((void*)v.value);   break;
            case   VTYPE_DICT: dict_free((void*)v.value);   break;
        }

        v = random_value_by_type(v.type, 1);
    }

    switch (v.type) {
        default: break;
        case VTYPE_STRING: string_free((void*)v.value); break;
        case  VTYPE_ARRAY: array_free((void*)v.value);  break;
        case   VTYPE_LIST: list_free((void*)v.value);   break;
        case    VTYPE_MAP: map_free((void*)v.value);    break;
        case    VTYPE_SET: vset_free((void*)v.value);   break;
        case   VTYPE_DICT: dict_free((void*)v.value);   break;
    }

    return x;
}

static set_t random_set(bool embd) {
    set_t   x;
    size_t  n = random_uint16()%((!embd)?MAX_ELEMENTS:100);
    value_t v = (!embd) ? random_value2() : real_random_value(1);

    vset_init(&x, v.type);

    while(n--) {

        libcdsb_vset_insert(&x, v.value, v.type);

        switch (v.type) {
            default: break;
            case VTYPE_STRING: string_free((void*)v.value); break;
            case  VTYPE_ARRAY: array_free((void*)v.value);  break;
            case   VTYPE_LIST: list_free((void*)v.value);   break;
            case    VTYPE_MAP: map_free((void*)v.value);    break;
            case    VTYPE_SET: vset_free((void*)v.value);   break;
            case   VTYPE_DICT: dict_free((void*)v.value);   break;
        }

        v = random_value_by_type(v.type, 1);
    }

    switch (v.type) {
        default: break;
        case VTYPE_STRING: string_free((void*)v.value); break;
        case  VTYPE_ARRAY: array_free((void*)v.value);  break;
        case   VTYPE_LIST: list_free((void*)v.value);   break;
        case    VTYPE_MAP: map_free((void*)v.value);    break;
        case    VTYPE_SET: vset_free((void*)v.value);   break;
        case   VTYPE_DICT: dict_free((void*)v.value);   break;
    }

    return x;
}

static list_t random_list(bool embd) {
    list_t  x;
    value_t v;
    size_t  n = random_uint16()%((!embd)?MAX_ELEMENTS:100);

    list_init(&x);

    while(n--) {
        v = (!embd) ? random_value2() : real_random_value(1);
        libcdsb_list_insert(&x, -1, v.value, v.type, 1);

        switch (v.type) {
            default: break;
            case VTYPE_STRING: string_free((void*)v.value); break;
            case  VTYPE_ARRAY: array_free ((void*)v.value); break;
            case   VTYPE_LIST: list_free  ((void*)v.value); break;
            case    VTYPE_MAP: map_free   ((void*)v.value); break;
            case    VTYPE_SET: vset_free  ((void*)v.value); break;
            case   VTYPE_DICT: dict_free  ((void*)v.value); break;
        }
    }

    return x;
}

static map_t random_map(bool embd) {
    map_t   x;
    size_t  n = random_uint16()%((!embd)?MAX_ELEMENTS:100);
    value_t k = (!embd) ? random_value2() : real_random_value(1);

    map_init(&x, k.type);

    while(n--) {
        value_t v = (!embd) ? random_value2() : real_random_value(1);

        libcdsb_map_update(&x, k.value, k.type, v.value, v.type);

        switch (k.type) {
            default: break;
            case VTYPE_STRING: string_free((void*)k.value); break;
            case  VTYPE_ARRAY: array_free((void*)k.value);  break;
            case   VTYPE_LIST: list_free((void*)k.value);   break;
            case    VTYPE_MAP: map_free((void*)k.value);    break;
            case    VTYPE_SET: vset_free((void*)k.value);   break;
            case   VTYPE_DICT: dict_free((void*)k.value);   break;
        }

        switch (v.type) {
            default: break;
            case VTYPE_STRING: string_free((void*)v.value); break;
            case  VTYPE_ARRAY: array_free ((void*)v.value); break;
            case   VTYPE_LIST: list_free  ((void*)v.value); break;
            case    VTYPE_MAP: map_free   ((void*)v.value); break;
            case    VTYPE_SET: vset_free  ((void*)v.value); break;
            case   VTYPE_DICT: dict_free  ((void*)v.value); break;
        }

        k = random_value_by_type(k.type, 1);
    }

    switch (k.type) {
        default: break;
        case VTYPE_STRING: string_free((void*)k.value); break;
        case  VTYPE_ARRAY: array_free((void*)k.value);  break;
        case   VTYPE_LIST: list_free((void*)k.value);   break;
        case    VTYPE_MAP: map_free((void*)k.value);    break;
        case    VTYPE_SET: vset_free((void*)k.value);   break;
        case   VTYPE_DICT: dict_free((void*)k.value);   break;
    }

    return x;

}

static dict_t random_dict(bool embd) {
    dict_t  x;
    value_t k, v;
    size_t  n = random_uint16()%((!embd)?MAX_ELEMENTS:100);

    dict_init(&x);

    while(n--) {
        k = (!embd) ? random_value2() : real_random_value(1);
        v = (!embd) ? random_value2() : real_random_value(1);
        libcdsb_dict_update(&x, k.value, k.type, v.value, v.type);

        switch (v.type) {
            default: break;
            case VTYPE_STRING: string_free((void*)v.value); break;
            case  VTYPE_ARRAY: array_free ((void*)v.value); break;
            case   VTYPE_LIST: list_free  ((void*)v.value); break;
            case    VTYPE_MAP: map_free   ((void*)v.value); break;
            case    VTYPE_SET: vset_free  ((void*)v.value); break;
            case   VTYPE_DICT: dict_free  ((void*)v.value); break;
        }

        switch (k.type) {
            default: break;
            case VTYPE_STRING: string_free((void*)k.value); break;
            case  VTYPE_ARRAY: array_free ((void*)k.value); break;
            case   VTYPE_LIST: list_free  ((void*)k.value); break;
            case    VTYPE_MAP: map_free   ((void*)k.value); break;
            case    VTYPE_SET: vset_free  ((void*)k.value); break;
            case   VTYPE_DICT: dict_free  ((void*)k.value); break;
        }
    }

    return x;
}

value_t random_container(bool embd) {
    value_t v;

    switch (random_uint8()%5) {
        default:
        case  0: v.value[0].a  = random_array(embd); v.type = VTYPE_ARRAY;  break;
        case  1: v.value[0].m  = random_map  (embd); v.type = VTYPE_MAP;    break;
        case  2: v.value[0].vd = random_dict (embd); v.type = VTYPE_DICT;   break;
        case  3: v.value[0].l  = random_list (embd); v.type = VTYPE_LIST;   break;
        case  4: v.value[0].vs = random_set  (embd); v.type = VTYPE_SET;    break;
    }

    return v;
}

value_t real_random_value(bool embd) {
    return random_boolean() ? random_value2() : random_container(embd);
}
