/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../plug.h"

static int list_node_print(void* v, ssize_t i, vtype t, void* _) {
    print_container_value(&i, v, t, 1, *(unsigned int*)_);
    return 0;
}

void list_print(list_t* x, const char* prefix, unsigned int hpos) {
    print_container_values_prefix("List", prefix, hpos);
    list_foreach(x, &hpos, list_node_print);
}

void list_info(list_t* x, unsigned int hpos) {
    print_container_info("List", "nodes", 0, list_size(x), -1, hpos);
}

void visual_push(list_t* x, size_t n) {
    for (int i = 0; i < n; ++i) {
        fputs("\e[s", stdout);

        list_push_random(x, 0, 0);

        list_info(x, 0);
        list_print(x, 0, 0);

        psleep(100000);
        fputs("\e[u\e[J", stdout);
    }
}

void visual_sort(list_t* x) {
    puts("\e[s\e[36mTry to sort list values:\e[m\n\n");
    put_separator(0);
    list_sort(x);
    list_info(x, 0);
    list_print(x, "sorted", 0);
    psleep(900000);
    fputs("\e[u\e[J", stdout);

    puts("\e[s\e[36mTry to reverse list values:\e[m\n\n");
    put_separator(0);
    list_reverse(x);
    list_info(x, 0);
    list_print(x, "reversed", 0);
    psleep(900000);
    fputs("\e[u\e[J", stdout);
}

void visual_remove(list_t* x) {
    while (x->first) {
        fputs("\e[s", stdout);
        list_remove_random(x, 0, 0);
        list_info(x, 0);
        list_print(x, 0, 0);

        psleep(100000);
        fputs("\e[u\e[J", stdout);
    }
}

void visual_push2(list_t* x0, size_t n0, list_t* x1, size_t n1) {

    for (;n0 || n1;) {
        fputs("\e[s", stdout);
        if (n0) {
            list_push_random(x0, 0, 0);
            --n0;
        } else {
            puts("\n\n");
            put_separator(0);
        }

        fputs("\e[u\e[s", stdout);

        if (n1) {
            list_push_random(x1, 0, 60);
            --n1;
        } else {
            puts("\n\n");
            put_separator(60);
        }

        list_info(x0, 0);
        list_print(x0, 0, 0);
        fputs("\e[u\e[s\e[4E", stdout);
        list_info(x1, 60);
        list_print(x1, 0, 60);
        psleep(100000);

        fputs("\e[u\e[J", stdout);
    }
}

void visual_remove2(list_t* x0, list_t* x1) {

    for (;x0->first || x1->first;) {
        fputs("\e[s", stdout);

        if (x0->first) {
            list_remove_random(x0, 0, 0);
        } else {
            puts("\n\n");
            put_separator(0);
        }

        fputs("\e[u\e[s", stdout);

        if (x1->first) {
            list_remove_random(x1, 0, 60);
        } else {
            puts("\n\n");
            put_separator(60);
        }

        list_info(x0, 0);
        list_print(x0, 0, 0);
        fputs("\e[u\e[s\e[4E", stdout);
        list_info(x1, 60);
        list_print(x1, 0, 60);
        psleep(100000);

        fputs("\e[u\e[J", stdout);
    }
}


void visual_sort2(list_t* x0, list_t* x1) {
    printf("\e[s\e[36m%-60s%s\e[m\n\n","Try to sort list values:", "Try to sort list values:");
    printf("\e[32;1m%-60s%s\e[m\n", "SUCCESS", "SUCCESS");
    printf("\e[37;2m%-60s%s\e[m\n", "=== === === === === === === ===", "=== === === === === === === ===");

    list_sort(x0);
    list_sort(x1);

    list_info(x0, 0);
    list_print(x0, "sorted", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    list_info(x1, 60);
    list_print(x1, "sorted", 60);

    psleep(900000);
    fputs("\e[u\e[J", stdout);

    printf("\e[s\e[36m%-60s%s\e[m\n\n","Try to reverse list values:", "Try to reverse list values:");
    printf("\e[32;1m%-60s%s\e[m\n", "SUCCESS", "SUCCESS");
    printf("\e[37;2m%-60s%s\e[m\n", "=== === === === === === === ===", "=== === === === === === === ===");

    list_reverse(x0);
    list_reverse(x1);

    list_info(x0, 0);
    list_print(x0, "reversed", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    list_info(x1, 60);
    list_print(x1, "reversed", 60);

    psleep(900000);
    fputs("\e[u\e[J", stdout);
}

void visual_extend(list_t* x, list_t* s) {

    puts("\e[s\e[36mTry to extend list:\e[m\n\n");
    printf("\e[37;2m%-60s%s\e[m\n", "=== === === === === === === ===", "=== === === === === === === ===");

    list_info(x, 0);
    list_print(x, "(dest)", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    list_info(s, 60);
    list_print(s, "(src)", 60);

    psleep(900000);
    fputs("\e[u\e[s\e[2E\e[32;1mSUCCESS\e[m\e[J", stdout);

    list_extend(x, s);

    puts("");
    put_separator(0);

    list_info(x, 0);
    list_print(x, "(dest)", 0);
    psleep(900000);
    fputs("\e[u\e[J", stdout);
}


void visual_slice(list_t* x, list_t* s) {

    size_t  n = list_size(x);
    ssize_t i = random_uint64()%(n - 1);

    n = n - (i + 1);

    puts("\e[s\e[36mTry to slice list:\e[m\n\n");
    printf("\e[37;2m%-60s%s\e[m\n", "=== === === === === === === ===", "=== === === === === === === ===");

    list_info(x, 0);
    list_print(x, "(src)", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    list_info(s, 60);
    list_print(s, "(dest)", 60);

    psleep(900000);

    list_slice(s, x, i, n, 1);
    fputs("\e[u\e[s\e[2E\e[32;1mSUCCESS\e[m\e[J", stdout);

    puts("");
    put_separator(0);

    list_info(x, 0);
    list_print(x, "(src)", 0);
    fputs("\e[u\e[s\e[4E", stdout);
    list_info(s, 60);
    list_print(s, "(dest)", 60);
    psleep(900000);

    fputs("\e[u\e[J", stdout);
}
