/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../plug.h"

static int remove_callback(void* v, ssize_t i, vtype t, void* _) {
    struct { list_t* x; _Bool s; unsigned int p; } *x = _;
    if (!x->s) {
        print_container_value(0, v, t, 1, x->p);
    }

    if (libcdsb_list_find(x->x, v, t, 0, 0, 1, 1)) {
        return -2;
    }

    return 0;
}


void list_push_random(list_t* x, _Bool silent, unsigned int hpos) {

    value_t v = random_value();
    _Bool   r;

    if (random_boolean()) {
        if (!silent) {
            printf("\e[%dG\e[36mTry to push value to back of the list:\e[m\n", hpos+1);
        }
        r = libcdsb_list_insert(x, -1, v.value, v.type,  1);
    } else if (random_boolean()) {
        if (!silent) {
            printf("\e[%dG\e[36mTry to push value to front of the list:\e[m\n", hpos+1);
        }
        r = libcdsb_list_insert(x, -1, v.value, v.type,  1);
    } else {
        ssize_t i = list_size(x);
        i = random_uint64()% (i ? i : 1);
        if (random_boolean()) i = ~i + 1;

        if (!silent) {
            printf("\e[%dG\e[36mTry to change value with index \e[32;1m%ld\e[36m in the list:\e[m\n", hpos+1, i);
        }

        r = libcdsb_list_insert(x, i, v.value, v.type, 0);
    }

    if (!silent) {
        print_container_value(0, v.value, v.type, 1, hpos);
        printf("\e[%dG%s\n", hpos+1, r ? "\e[32;1mSUCCESS\e[m" : "\e[31;1mFAILURE\e[m");
        put_separator(hpos);
    }
}

void list_remove_random(list_t* x, _Bool silent, unsigned int hpos) {

    size_t  n = list_size(x);
    ssize_t i = random_uint64()%n;

    if (random_boolean()) i = ~i + 1;

    if (random_boolean()) {
        if (!silent) {
            printf("\e[%dG\e[36mTry to remove value from list by index:\e[m\n", hpos+1);
            print_container_value(0, &i, (sizeof(ssize_t) == 8) ? VTYPE_INT64 : VTYPE_INT32, 0, hpos);
        }
        switch (list_remove_by_index(x, i)) {
            case  0: if (!silent) printf("\e[%dG\e[32;1mSUCCESS\e[m\n", hpos+1); break;
            default: if (!silent) printf("\e[%dG\e[32;1mFAILURE\e[m\n", hpos+1); break;
        }
    } else {
        struct { list_t* x; _Bool s; unsigned int p; } v = { .x = x, .s = silent, .p = hpos };

        if (!silent) printf("\e[%dG\e[36mTry to remove value from list:\e[m\n", hpos+1);

        switch (libcdsb_list_get(x, i, &v, remove_callback, 0)) {
            case  0: if (!silent) printf("\e[%dG\e[32;1mSUCCESS\e[m\n", hpos+1); break;
            default: if (!silent) printf("\e[%dG\e[32;1mFAILURE\e[m\n", hpos+1); break;
        }
    }

    if (!silent) put_separator(hpos);
}
