/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../plug.h"

static int node_print_callback(const void* k, vtype kt, void* v, vtype vt, void* _) {
    print_container_value(0, k, kt, 0, *(unsigned int*)_);
    return 0;
}

static void rbtree_print(const mnode_t* s, vtype t, const char* ind, bool br, unsigned int hpos) {
    if (!ind) {
        ind = "\e[36m";
        br  = 1;
    }

    size_t n = strlen(ind);
    char x[n + 10];

    if (mnode_is_empty(s)) return;

    memcpy(x, ind, n);
    memcpy(x + n, "     \0  ", 9);

    printf("\e[%dG%s", hpos+1, ind);

    if (br) {
        fputs("\e[m\e[36;1mR\e[m\e[36m────\e[m", stdout);
    } else {
        fputs("\e[m\e[36;1mL\e[m\e[36m────\e[m", stdout);
        memcpy(x + n, "│", 3);
        x[n + 5] = ' ';
    }

    fputs((s->colored) ? "\e[31;1m" : "\e[37m", stdout);
    printf("%s\e[m \e[36m:\e[m ", vnode_stringify(&s->key, t));
    printf("\e[31m%s\e[m", vnode_stringify(&s->value, s->type));
    printf(" \e[36m(\e[m\e[32;1m%s\e[m\e[36m)\e[m\n", vtype_name(s->type));

    rbtree_print(s->left,  t, x, false, hpos);
    rbtree_print(s->right, t, x, true, hpos);
}

void map_print(map_t* x, const char* prefix, unsigned int hpos) {
    print_container_values_prefix("Map", prefix, 0);
    map_foreach(x, &hpos, node_print_callback);
    put_separator(0);
}

void map_info(const map_t* x, unsigned int hpos) {
    print_container_info("Map", "nodes", &x->type, map_size(x), -1, 0);
    put_separator(0);
}

void map_rbtree_print(map_t *x, const char* prefix, unsigned int hpos) {
    print_container_values_prefix("Map", prefix, 0);
    rbtree_print(x->root, x->type, 0, 0, hpos);
}


void visual_push(map_t* x, size_t n) {
    while (n--) {
        fputs("\e[s", stdout);

        map_push_random(x, 0, 0);

        map_info(x, 0);
        map_rbtree_print(x, 0, 0);

        psleep(100000);
        fputs("\e[u\e[J", stdout);
    }
}

void visual_remove(map_t* x) {
    while (!mnode_is_empty(x->root)) {
        fputs("\e[s", stdout);
        map_remove_random(x, 0, 0);
        map_info(x, 0);
        map_rbtree_print(x, 0, 0);
        psleep(100000);
        fputs("\e[u\e[J", stdout);
    }
}
