/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "../plug.h"

static int remove_callback(const void* v, vtype t, void* _) {
    struct { size_t n; set_t* x; unsigned int hp; } *d = _;

    if (!d->n--) {
        print_container_value(0, v, t, 0, d->hp);

        if (libcdsb_vset_find(d->x, v, t, 0, 0, 1) == 0) {
            printf("\e[%dG\e[32;1mSUCCESS\e[m\n", d->hp+1);
        } else printf("\e[%dG\e[31;1mFAILURE\e[m\n", d->hp+1);

        return -2;
    }
    return 0;
}

void vset_push_random(set_t* x, _Bool silent, unsigned int hpos) {

    value_t v = random_value();

    if (!silent) {
        printf("\e[%dG\e[36mUpdate value in set (\e[m\e[32;1m%s\e[m\e[36m):\e[m\n", hpos+1, vtype_name(x->type));
        print_container_value(0, v.value, v.type, 1, hpos);
    }

    if (libcdsb_vset_insert(x, v.value, v.type)) {
        if (!silent) printf("\e[%dG\e[32;1mSUCCESS\e[m\n", hpos+1);
    } else if (!silent) printf("\e[%dG\e[31;1mFAILURE\e[m\n", hpos+1);

    if (!silent) put_separator(0);
}


void vset_remove_random(set_t* x, _Bool silent, unsigned int hpos) {
    struct { size_t n; set_t* x; unsigned int hp; } d = { .n = vset_size(x), .x = x, .hp = hpos };

    if (!silent)
        printf("\e[%dG\e[36mTry to remove value from set (\e[m\e[32;1m%s\e[m\e[36m):\e[m\n", hpos+1, libcdsb_vtype_name(x->type));

    if (d.n) {
        d.n = random_uint32()%d.n;
        vset_foreach(x, &d, remove_callback);
    } else if (!silent) {
        printf("\e[%dG\e[32;1m\nFAILURE\e[m\n", hpos+1);
    }

    if (!silent) put_separator(hpos);
}
